%global includetests 0
# 0=no, 1=yes
%global cryptlibdir %{_libdir}/%{name}

Name:       cryptlib
Version:    3.4.3  
Release:    1%{?dist}
Summary:    Security library and toolkit for encryption and authentication services    

Group:      System Environment/Libraries         
License:    Sleepycat       
URL:        https://www.cs.auckland.ac.nz/~pgut001/cryptlib      
Source0:    https://crypto-bone.com/fedora/cl343_fedora.zip      
Source1:    https://crypto-bone.com/fedora/cl343_fedora.zip.sig
Source2:    gpgkey-3274CB29956498038A9C874BFBF6E2C28E9C98DD.asc
Source3:    http://www.cypherpunks.to/~peter/manual.pdf
Source4:    cryptlib-tests.tar.gz

Patch1:     sonamepatch
# soname is now libcl.so.3
Patch2:     ccflagspatch
Patch3:     sessionpatch
Patch4:     utilspatch
Patch5:     stackprotectorstrongpatch
Patch6:     javapatch

ExclusiveArch: x86_64 %{ix86} %{arm}

BuildRequires: gcc 
BuildRequires: libbsd-devel   
BuildRequires: gnupg2
BuildRequires: python >= 2.7
BuildRequires: python2-devel >= 2.7
#BuildRequires: python3-devel
BuildRequires: java-devel


Provides: libcl.so.3 = 3.4.3


%description
Cryptlib is a powerful security toolkit that allows even inexperienced crypto
programmers to easily add encryption and authentication services to their
software. The high-level interface provides anyone with the ability to add
strong security capabilities to an application in as little as half an hour,
without needing to know any of the low-level details that make the encryption
or authentication work.  Because of this, cryptlib dramatically reduces the
cost involved in adding security to new or existing applications.

At the highest level, cryptlib provides implementations of complete security
services such as S/MIME and PGP/OpenPGP secure enveloping, SSL/TLS and
SSH secure sessions, CA services such as CMP, SCEP, RTCS, and OCSP, and other
security operations such as secure time-stamping. Since cryptlib uses
industry-standard X.509, S/MIME, PGP/OpenPGP, and SSH/SSL/TLS data formats,
the resulting encrypted or signed data can be easily transported to other
systems and processed there, and cryptlib itself runs on virtually any
operating system - cryptlib doesn't tie you to a single system.
This allows email, files and EDI transactions to be authenticated with
digital signatures and encrypted in an industry-standard format.


%package devel
Summary:  Cryptlib application development files 
Group:    System Environment/Libraries

%description devel
Header files and code for application development in C (and C++)


%package doc
Summary:  Manual for application development with cryptlib 

%description doc
A comprehensive software developer's guide to using cryptlib. 

The manual provides code examples in C, Java, Python and other
languages and detailed descriptions of the cryptlib security
architecture, including the explanation of its High-, Medium-
and Low-level interface. All information needed to add security
services to existing applications is easily accessible with this
manual.


%package test
Summary:  Cryptlib test program
Requires: %{name}%{?_isa} = %{version}-%{release}

%description test
Cryptlib test programs for C, Java, Perl and Python


%package java
Summary:  Cryptlib bindings for Java
Group:    System Environment/Libraries
Requires: %{name}%{?_isa} = %{version}-%{release}
Requires: java-headless

%description java
Cryptlib module for application development in Java

%package python2
Summary:  Cryptlib bindings for python2
Group:    System Environment/Libraries
Requires: %{name}%{?_isa} = %{version}-%{release}
Requires: python >= 2.7
Provides: cryptlib_py.so

%description python2
Cryptlib module for application development in Python


# The python3 subpackage cannot be build, because DL_EXPORT is missing in Python.h
# so python3 setup.py build fails
#
#%package python3
#Summary:  Cryptlib bindings for python3
#Group:    System Environment/Libraries
#
#%description python3
#Cryptlib module for application development in Python



%prep
# source code signature check with GnuPG
KEYRING=$(echo %{SOURCE2})
KEYRING=${KEYRING%%.asc}.gpg
mkdir -p .gnupg
gpg2 --homedir .gnupg --no-default-keyring --quiet --yes --output $KEYRING --dearmor  %{SOURCE2}
gpg2 --homedir .gnupg --no-default-keyring --keyring $KEYRING --verify %{SOURCE1} %{SOURCE0}

rm -rf %{name}-%{version}
mkdir %{name}-%{version}
cd %{name}-%{version}
/usr/bin/unzip -a %{SOURCE0}
%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%patch6 -p1
# remove pre-build jar file
rm %{_builddir}/%{name}-%{version}/bindings/cryptlib.jar


%build
cd %{name}-%{version}
# build java bindings
chmod +x tools/mkhdr.sh
tools/mkhdr.sh
cp /etc/alternatives/java_sdk/include/jni.h .
cp /etc/alternatives/java_sdk/include/linux/jni_md.h .

make clean
make shared %{?_smp_mflags} ADDFLAGS="%{optflags}"
make stestlib %{?_smp_mflags} ADDFLAGS="%{optflags}"

# build python modules
ln -s libcl.so.3.4.3 libcl.so
cd bindings
python2 setup.py build
# DL_EXPORT is missing in Python.h, so the following build fails.
#python3 setup.py build


%install
mkdir -p %{buildroot}%{_libdir}
mkdir -p %{buildroot}%{_datadir}/licenses/%{name}
mkdir -p %{buildroot}%{_docdir}/%{name}
cp %{_builddir}/%{name}-%{version}/libcl.so.3.4.3 %{buildroot}%{_libdir}
cd %{buildroot}%{_libdir}
ln -s libcl.so.3.4.3 libcl.so.3
ln -s libcl.so.3 libcl.so

# install header files
mkdir -p %{buildroot}/%{_includedir}/%{name}
cp %{_builddir}/%{name}-%{version}/crypt.h %{buildroot}%{_includedir}/%{name}
cp %{_builddir}/%{name}-%{version}/cryptkrn.h %{buildroot}%{_includedir}/%{name}
cp %{_builddir}/%{name}-%{version}/cryptlib.h %{buildroot}%{_includedir}/%{name}

# add Java bindings
mkdir -p %{buildroot}/%{cryptlibdir}/java
mkdir -p %{buildroot}/%{_jnidir}
cp %{_builddir}/%{name}-%{version}/bindings/cryptlib.jar %{buildroot}%{cryptlibdir}/java
cp %{_builddir}/%{name}-%{version}/bindings/cryptlib.jar %{buildroot}%{_jnidir}

# install docs
cp %{_builddir}/%{name}-%{version}/COPYING %{buildroot}%{_datadir}/licenses/%{name}
cp %{_builddir}/%{name}-%{version}/README %{buildroot}%{_docdir}/%{name}/README
echo "No tests performed." > %{_builddir}/%{name}-%{version}/stestlib.log
cp %{_builddir}/%{name}-%{version}/stestlib.log %{buildroot}%{_docdir}/%{name}/stestlib.log
cp %{SOURCE3} %{buildroot}%{_docdir}/%{name}/manual.pdf

# install python2 module
mkdir -p %{buildroot}%{python2_sitelib}
cp %{_builddir}/%{name}-%{version}/bindings/build/lib.linux-*%{python2_version}/cryptlib_py.so %{buildroot}%{python2_sitelib}

# install test programs
cp %{_builddir}/%{name}-%{version}/stestlib %{buildroot}%{cryptlibdir}
cp -r %{_builddir}/%{name}-%{version}/test %{buildroot}%{cryptlibdir}/test
# remove all c code from the test directory
rm -rf $(find %{buildroot}%{cryptlibdir}/test -name "*.c")

## remove all header files from the test directory
# these header files are needed by the test program stestlib to find test files!
#rm -rf $(find %{buildroot}%{cryptlibdir}/test -name "*.h")

cd %{buildroot}%{cryptlibdir}
tar xpzf %{SOURCE4} 

%check
# checks are performed after install
%if %{includetests}
     cd %{_builddir}/%{name}-%{version}
     ln -s libcl.so.3.4.3 ./libcl.so.3
     export LD_LIBRARY_PATH=.
     echo "Running tests on the cryptlib library. This will take a few minutes."
     echo "Network access is necessary to complete all tests!"
     ./stestlib > %{_builddir}/%{name}-%{version}/stestlib.log
     cp %{_builddir}/%{name}-%{version}/stestlib.log %{buildroot}%{_docdir}/%{name}/stestlib.log
%endif


%post -p /sbin/ldconfig

%postun -p /sbin/ldconfig


%files
%{_libdir}/libcl.so.3.4.3
%{_libdir}/libcl.so.3

%license   %{_datadir}/licenses/%{name}/COPYING
%doc       %{_docdir}/%{name}/README
%doc       %{_docdir}/%{name}/stestlib.log


%files devel
%{_libdir}/libcl.so
%{_includedir}/%{name}/crypt.h
%{_includedir}/%{name}/cryptkrn.h
%{_includedir}/%{name}/cryptlib.h

%files java
%{_libdir}/libcl.so
%{cryptlibdir}/java/cryptlib.jar
%{_jnidir}/cryptlib.jar

%files python2
%{python2_sitelib}/cryptlib_py.so

# the python3 subpackage cannot be build
#%files python3
#%{python3_sitelib}/cryptlib_py.so

%files doc
%doc       %{_docdir}/%{name}/manual.pdf

%files test
%{cryptlibdir}


%changelog
* Wed Jun 1 2016 Senderek Web Security <innovation@senderek.ie> - 3.4.3-1
- Added python2/python3 subpackage
- Source code signature check with GnuPG enabled

* Sun May 29 2016 Senderek Web Security <innovation@senderek.ie> - 3.4-2
- Added doc and java subpackage

* Fri May 27 2016 Senderek Web Security <innovation@senderek.ie> - 3.4-1
- Initial version of the rpm package build
